from distutils.core import setup
setup(
  name = 'veripy',
  packages = ['veripy'], # this must be the same as the name above
  version = '0.2',
  description = 'Shorthand asserts for those who love predictable code but dont have time for writing every assert in the world.',
  author = 'Cody Kochmann',
  author_email = 'kochmanncody@gmail.com',
  url = 'https://github.com/CodyKochmann/veripy',
  download_url = 'https://github.com/CodyKochmann/veripy/tarball/0.2',
  keywords = ['assert', 'shorthand', 'contract', 'require', 'veripy'], # arbitrary keywords
  classifiers = [],
)
